//+------------------------------------------------------------------+
//|                                     CCI_Divergence.mq4           |
//+------------------------------------------------------------------+
#property copyright 
//----
#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1 Green
#property indicator_color2 Red
#property indicator_color3 Magenta
//----
#define arrowsDisplacement 2
extern int cciPeriod = 14;

extern bool   showClassicalDivergence=false;
extern bool   showHiddenDivergence=true;

extern bool   drawIndicatorTrendLines = true;
extern bool   drawPriceTrendLines = true;
extern bool   displayAlert = true;

//---- buffers
double bullishDivergence[];
double bearishDivergence[];
double cci[];
//----
static datetime lastAlertTime;

int init()
{
//---- indicators
   SetIndexStyle(0, DRAW_ARROW);
   SetIndexStyle(1, DRAW_ARROW);
   SetIndexStyle(2, DRAW_LINE);
//----   
   SetIndexBuffer(0, bullishDivergence);
   SetIndexBuffer(1, bearishDivergence);
   SetIndexBuffer(2, cci); 
//----   
   SetIndexArrow(0, 233);
   SetIndexArrow(1, 234);
//----
   IndicatorDigits(MarketInfo(Symbol(),MODE_DIGITS)+2);
   IndicatorShortName("CCI_Divergence(" + cciPeriod + ")");
//----
   return(0);
}


int deinit()
  {
   for(int i = ObjectsTotal() - 1; i >= 0; i--)
     {
       string label = ObjectName(i);
       if(StringSubstr(label, 0, 18) != "CCI_DivergenceLine")
           continue;
       ObjectDelete(label);   
     }
   return(0);
  }

int start()
  {
   int countedBars = IndicatorCounted();
   if(countedBars < 0)
       countedBars = 0;
   calculateIndicator(countedBars);
   return(0);
  }

void calculateIndicator(int countedBars)
{
   for (int i = Bars - countedBars; i >= 0; i--)
   {
      calculateCCI(i);
      catchBullishDivergence(i+1);
      catchBearishDivergence(i+1);
   }              
}

void calculateCCI(int i)
{
   cci[i] = iCCI(NULL,0,cciPeriod,PRICE_CLOSE,i);       
}

void catchBullishDivergence(int shift)
{
   if(!isIndicatorTrough(shift))
       return;  
   int currentTrough = shift;
   int lastTrough = getIndicatorLastTrough(shift);
   
   if(lastTrough==-1)
      return;
   if(showClassicalDivergence && cci[currentTrough] > cci[lastTrough] && 
      Low[currentTrough] < Low[lastTrough])
   {
       bullishDivergence[currentTrough] = cci[currentTrough] - 
                                          arrowsDisplacement;
       if(drawPriceTrendLines == true)
           drawPriceTrendLine(Time[currentTrough], Time[lastTrough], 
                              Low[currentTrough], 
                              Low[lastTrough], Yellow, STYLE_SOLID);
       if(drawIndicatorTrendLines == true)
           drawIndicatorTrendLine(Time[currentTrough], 
                                  Time[lastTrough], cci[currentTrough],
                                  cci[lastTrough], Yellow, STYLE_SOLID);
       if(displayAlert)
           displayAlert("Classical bullish divergence on: ", 
                        currentTrough);  
   }
   if(showHiddenDivergence && cci[currentTrough] < cci[lastTrough] && 
      Low[currentTrough] > Low[lastTrough])
   {
       bullishDivergence[currentTrough] = cci[currentTrough] - 
                                          arrowsDisplacement;
       if(drawPriceTrendLines == true)
           drawPriceTrendLine(Time[currentTrough], Time[lastTrough], 
                              Low[currentTrough], 
                              Low[lastTrough], Yellow, STYLE_DOT);
       if(drawIndicatorTrendLines == true)                            
           drawIndicatorTrendLine(Time[currentTrough], Time[lastTrough], 
                                  cci[currentTrough],
                                  cci[lastTrough], Yellow, STYLE_DOT);
       if(displayAlert == true)
           displayAlert("Reverse bullish divergence on: ", currentTrough);   
   }      
}

void catchBearishDivergence(int shift)
{
   if(!isIndicatorPeak(shift))
      return;
   int currentPeak = shift;
   int lastPeak = getIndicatorLastPeak(shift);
   
   if(lastPeak==-1)
      return;
   if(showClassicalDivergence && cci[currentPeak] < cci[lastPeak] && 
      High[currentPeak] > High[lastPeak])
   {
      bearishDivergence[currentPeak] = cci[currentPeak] + 
                                        arrowsDisplacement;
      if(drawPriceTrendLines == true)
          drawPriceTrendLine(Time[currentPeak], Time[lastPeak], 
                              High[currentPeak], 
                              High[lastPeak], Red, STYLE_SOLID);                   
      if(drawIndicatorTrendLines == true)
           drawIndicatorTrendLine(Time[currentPeak], Time[lastPeak], 
                                  cci[currentPeak],
                                  cci[lastPeak], Red, STYLE_SOLID);
      if(displayAlert == true)
           displayAlert("Classical bearish divergence on: ", currentPeak);  
   }
   if(showHiddenDivergence && cci[currentPeak] > cci[lastPeak] && 
      High[currentPeak] < High[lastPeak])
   {
      bearishDivergence[currentPeak] = cci[currentPeak] + 
                                        arrowsDisplacement;
      if (drawPriceTrendLines == true)
         drawPriceTrendLine(Time[currentPeak], Time[lastPeak], 
                              High[currentPeak], 
                              High[lastPeak], Red, STYLE_DOT);
      if (drawIndicatorTrendLines == true)
         drawIndicatorTrendLine(Time[currentPeak], Time[lastPeak], 
                                  cci[currentPeak],
                                  cci[lastPeak], Red, STYLE_DOT);
      if (displayAlert == true)
           displayAlert("Reverse bearish divergence on: ", currentPeak);   
   }   
}

bool isIndicatorPeak(int shift)
  {
   if(cci[shift] >= cci[shift+1] && cci[shift] > cci[shift+2] && 
      cci[shift] > cci[shift-1])
       return(true);
   else 
       return(false);
  }

bool isIndicatorTrough(int shift)
{
   if(cci[shift] <= cci[shift+1] && cci[shift] < cci[shift+2] && 
      cci[shift] < cci[shift-1])
       return(true);
   else 
       return(false);
}

int getIndicatorLastPeak(int shift)
{
   for(int index=1;index<5;index++)
   {
      if(cci[shift+index]>cci[shift])
         return(-1);
      if(cci[shift+index]>cci[shift+index+1] && cci[shift+index]>cci[shift+index-1])
         return(-1);
   }
   for(int i = shift+5; i < Bars; i++)
   {
      if(cci[i] >= cci[i+1] && cci[i] > cci[i+2] && cci[i] >= cci[i-1] && cci[i] > cci[i-2])
         return(i);
   }
   return(-1);
}


int getIndicatorLastTrough(int shift)
{
   for(int index=1;index<5;index++)
   {
      if(cci[shift+index]<cci[shift])
         return(-1);
      if(cci[shift+index]<cci[shift+index+1] && cci[shift+index]<cci[shift+index-1])
         return(-1);
   }
   for(int i = shift+5; i < Bars; i++)
   {
      if(cci[i] <= cci[i+1] && cci[i] <= cci[i+2] && cci[i] <= cci[i-1] && cci[i] <= cci[i-2])
      {
         return(i);
      }
   }
   return(-1);
}


void displayAlert(string message, int shift)
{
   if(shift <= 2 && Time[shift] != lastAlertTime)
   {
       lastAlertTime = Time[shift];
       Alert(message, Symbol(), " , ", Period(), " minutes chart");
   }
}


void drawPriceTrendLine(datetime x1, datetime x2, double y1, 
                        double y2, color lineColor, double style)
{
   string label = "CCI_DivergenceLine_v1.0# " + DoubleToStr(x1, 0);
   ObjectDelete(label);
   ObjectCreate(label, OBJ_TREND, 0, x1, y1, x2, y2, 0, 0);
   ObjectSet(label, OBJPROP_RAY, 0);
   ObjectSet(label, OBJPROP_COLOR, lineColor);
   ObjectSet(label, OBJPROP_STYLE, style);
}

void drawIndicatorTrendLine(datetime x1, datetime x2, double y1, 
                            double y2, color lineColor, double style)
{
   int indicatorWindow = WindowFind("CCI_Divergence(" + cciPeriod + ")");
   if(indicatorWindow < 0)
       return;
   string label = "CCI_DivergenceLine_v1.0$# " + DoubleToStr(x1, 0);
   ObjectDelete(label);
   ObjectCreate(label, OBJ_TREND, indicatorWindow, x1, y1, x2, y2, 0, 0);
   ObjectSet(label, OBJPROP_RAY, 0);
   ObjectSet(label, OBJPROP_COLOR, lineColor);
   ObjectSet(label, OBJPROP_STYLE, style);
}